//@jwriter

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;

public class Glonass {
    
    byte[] raw;
    
    private static int LENGTH = 1_000_000; 
    private static int BOARD = 12;
    private static int BUF_SZ = 100_000; 
    
    private static double SPEED = 299792458.0;
    
    private static int PLACE = 427_400;
    private static int BOTTOM = PLACE + 3_000_000/30;
    private static int TOP = PLACE + 27_000_000/30;
    
    private int N;
    private Record satelites[];
    
    private Point founded;
    
    public Glonass(String nameFile) {
        try{
            long before = System.currentTimeMillis();
            readFile(nameFile);
//            System.out.println("--read-> time: "
//                                + (System.currentTimeMillis() - before));
            parse();
//            System.out.println("--parse-> time: "
//                                + (System.currentTimeMillis() - before));
            calcDist();
//            System.out.println("--calcDist-> time: "
//                                + (System.currentTimeMillis() - before));
            founded = calcPoint();
//            System.out.println("--calcPoint-> time: "
//                                + (System.currentTimeMillis() - before));
            String str = founded.x + " " + founded.y;
//            System.out.println(str);
            makeOutputFile(nameFile, str);
        } catch(FileNotFoundException e) {
            System.out.println("ERROR - couldn't open file: " + nameFile);
        } catch(IOException e2) {
            System.out.println("ERROR - when read from file: " + nameFile);
        }
    }

    private void makeOutputFile(String nameFile, String str) {
        String newFile = nameFile.replace(".in", ".out");
        try (FileOutputStream fos = new FileOutputStream(newFile)){
            byte[] b = str.getBytes();
            fos.write(b);
            fos.flush();
        } catch (FileNotFoundException e) {
            
        } catch (IOException e) {

        };
        
    }

    private void readFile(String nf) throws IOException {
   
        File f = new File(nf);
        int sz = (int)f.length();
        
        raw = new byte[sz];
        FileInputStream fis = new FileInputStream(f);
        int cnt = 0;
        do {
            cnt += fis.read(raw, cnt, (cnt + BUF_SZ > sz)?sz - cnt:BUF_SZ);
        } while (cnt < f.length());

    }
    
    private void parse() {
        int cnt = 0; int begin = 0;
        while (raw[cnt++] != 10);
        N = Integer.valueOf(new String(raw, begin, cnt - 2));
        satelites = new Record[N];
        for(int i = 0; i < N; i ++) {
            satelites[i] = new Record();
            begin = cnt;
            while (raw[cnt++] != 10);
            satelites[i].x = Double.valueOf(new String(raw, begin , cnt - 1 - begin));
            begin = cnt ;
            while (raw[cnt++] != 10);
            satelites[i].y  = Double.valueOf(new String(raw, begin, cnt - begin - 1));
            satelites[i].begin = cnt;
            begin = cnt;
            byte tmp = raw[begin];
            while (raw[++begin] == tmp);
            satelites[i].shift = (begin - cnt) % 10; 
            cnt += 10_000_000 + 2;
        }
        
    }
      
    public static void main(String[] args) throws Exception {
        
        
        if (args.length < 1) {
            System.out.println("You must type name of data file in the "
                    + "next way: java Glonass your_data_file");
        } else {
            //System.out.println(args[0]);
            Glonass gl = new Glonass(args[0]);
           
        }
    }

    private Point calcPoint() {
        Point[] points = new Point[2 * (N - 1)];
        for (int i = 1; i < N; i++) {
            double x = satelites[i].x - satelites[0].x;
            double y = satelites[i].y - satelites[0].y;
            double R = satelites[0].dAvg();
            double r = satelites[i].dAvg();
            double D = dist(satelites[i].x, satelites[i].y, satelites[0].x, satelites[0].y);
            double cosA = (D * D + R * R - r * r)/(2 * D * R);
            double a = Math.acos(cosA);
            double k = Math.atan(y/x);
            if (x < 0) k = k + Math.PI;
            Point p = new Point();
            p.x = R * Math.cos(k - a) + satelites[0].x;
            p.y = R * Math.sin(k - a) + satelites[0].y;
            Point p1 = new Point();
            p1.x = R * Math.cos(k + a) + satelites[0].x;
            p1.y = R * Math.sin(k + a) + satelites[0].y;
            points[2 * (i - 1)] = p;
            points[2 * (i - 1) + 1] = p1;
        }
        if (N == 2) {
            if (checkPoint(points[0])){
                return points[0];
            } else return points[1];
        } else {
            
            Point res = findAveragePoint(points);
            
            //res = findMonteCarlo(res);
            return res;
        }
        
    }

    private Point findAveragePoint(Point[] p) {
        Point res;
        int repper = -1;
        boolean[] apr = new boolean[p.length];
        apr[0] = checkPoint(p[0]);
        apr[1] = checkPoint(p[1]);
        try{
            if (apr[0] && apr[1]) {
                try {
                    repper = 0;
                    res = findNearRepper(p, repper, apr);
                } catch (Exception e) {
                    repper = 1;
                    res = findNearRepper(p, repper, apr);
                }
            } else {
                if (apr[0]) repper = 0;
                else repper = 1;
                res = findNearRepper(p, repper, apr);
            }
        } catch (Exception e) {
            res = p[repper];
        }
        return res;
    }
    
    private Point findNearRepper(Point[] p, int repper, boolean[] apr)
                                throws Exception {
        ArrayList <Point> list = new ArrayList<>();
        list.add(p[repper]);
        for (int i = 1; i < N - 1; i++) {
            apr[2 * i] = checkPoint(p[2 * i]);
            apr[2 * i + 1] = checkPoint(p[2 * i + 1]);
            double d1 = dist(p[repper], p[2 * i]);
            double d2 = dist(p[repper], p[2 * i + 1]);
            if (d1 < 300) {
               list.add(p[2 * i]);
            }
            if (d2 < 300) {
                list.add(p[2 * i + 1]);
            }
            if (d1 > 1000 && d2 > 1000) throw new Exception();
            //if (!apr[2 * i] && !apr[2 * i + 1]) throw new Exception();
        }
        return averageList(list);
    }

    private Point averageList(ArrayList<Point> list) {
        double x = 0;
        double y = 0;
        for (int i = 0; i < list.size(); i++) {
            x += list.get(i).x;
            y += list.get(i).y;
        }
        Point res = new  Point();
        res.x = x/list.size();
        res.y = y/list.size();
        return res;
    }

    private  boolean checkPoint(Point point) {
        if (dist(point.x, point.y, 0, 0) < 6_378_000) return true; 
        return false;
    }

    private static double dist2(double x1, double y1, double x2, double y2) {
        return (x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 -y2);
    }
    
    private  double dist(double x1, double y1, double x2, double y2) {
        return Math.sqrt(dist2(x1, y1, x2, y2));
    }
    
    private  double dist(Point p1, Point p2) {
        return Math.sqrt(dist2(p1.x, p1.y, p2.x, p2.y));
    }
    
    
    private void calcDist() {

      for (int i = 0; i < N; i++) {
          
          int marker = findZeroMarker(i) - PLACE - satelites[i].begin  - satelites[i].shift;
          satelites[i].dMin = (marker  + satelites[i].shift) * SPEED/10_000_000.0;
          satelites[i].dMax = (marker  + satelites[i].shift + 1) * SPEED/10_000_000.0;
      }
    }

    private int findZeroMarker(int nS) {
        int beginPos = BOTTOM + satelites[nS].begin + satelites[nS].shift;
        int endPos = TOP + satelites[nS].begin + satelites[nS].shift;
        int pos = -1;
        int cnt = 0;
        int prev = raw[0];
        if (prev == (byte) '0') {
            pos = 0;
        }
        for (int i = beginPos; i < endPos; i = i + 10) {
            int now = raw[i];
            if (now == (byte) '0') {
                if (prev == (byte) '1') {
                    pos = i;
                }
                cnt++;
            } else {
                if (cnt == 18) {
                    return pos;
                }
                cnt = 0;
            }
            prev = now;
        }
        return -1;
    }
  
    class Record {
        double x;
        double y;
        
        int shift;
        int begin;
        double dMin;
        double dMax;
        
        double dAvg() {
            return (dMin + dMax)/2;
        }
        
        boolean isAppropriate(double xP, double yP) {
            double dReal = dist(x, y, xP, yP);
            if (dReal >= dMin && dReal <= dMax) return true;
            return false;
        }
    }
    
    class Point {
        double x;
        double y;
    }

}
